@@ -12,7 +12,7 @@
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: flighttask.cpp 4629 2011-05-14 19:22:48Z axel $
+**   $Id: flighttask.cpp 4675 2011-09-18 16:58:14Z axel $
 **
 **   This class provides calculation, drawing and other support
 **   for flight tasks and contains the data of a task.
@@ -785,16 +785,6 @@ void FlightTask::drawTask( QPainter* painter, QList<Waypoint*> &drawnTp )
  */
 void FlightTask::circleSchemeDrawing( QPainter* painter, QList<Waypoint*> &drawnTp )
 {
-  extern MapConfig* _globalMapConfig;
-
-  // determine icon size to be used
-  int iconSize = 32;
-
-  if ( _globalMapConfig->useSmallIcons() )
-    {
-      iconSize = 16;
-    }
-
   // get user defined scheme items
   GeneralConfig *conf = GeneralConfig::instance();
 
