@@ -12,7 +12,7 @@
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: generalconfig.h 4597 2011-04-28 18:55:45Z axel $
+**   $Id: generalconfig.h 4667 2011-09-09 20:42:26Z axel $
 **
 ***********************************************************************/
 
@@ -29,7 +29,7 @@
  *
  * \date 2004-2011
  *
- * \version $Id: generalconfig.h 4597 2011-04-28 18:55:45Z axel $
+ * \version $Id: generalconfig.h 4667 2011-09-09 20:42:26Z axel $
  */
 
 #ifndef GENERAL_CONFIG_H
@@ -1543,6 +1543,17 @@ class GeneralConfig : protected QSettings
   /** sets Gps sync system clock */
   void setGpsSyncSystemClock(const bool newValue);
 
+  /** gets Gps NMEA log state */
+  bool getGpsNmeaLogState() const
+  {
+    return _gpsNmeaLogState;
+  }
+  /** sets Gps NMEA log state */
+  void setGpsNmeaLogState(const bool newValue)
+  {
+    _gpsNmeaLogState = newValue;
+  }
+
   /** gets Gps Ipc port */
   ushort getGpsIpcPort() const;
   /** sets Gps Ipc port */
@@ -2266,6 +2277,8 @@ class GeneralConfig : protected QSettings
   bool _gpsHardStart;
   // Gps synchronize system clock
   bool _gpsSyncSystemClock;
+  // Gps NMEA log state
+  bool _gpsNmeaLogState;
   // Gps IPC port
   ushort _gpsIpcPort;
   // Gps client start option
