@@ -6,7 +6,7 @@
                                2008-2010 by Axel Pauli
     email                : axel@kflog.org
 
-    $Id: gpsnmea.h 4502 2010-12-09 22:32:02Z axel $
+    $Id: gpsnmea.h 4667 2011-09-09 20:42:26Z axel $
 
  ***************************************************************************
  *                                                                         *
@@ -41,6 +41,7 @@
 #include <QPoint>
 #include <QTimer>
 #include <QHash>
+#include <QFile>
 
 #include "speed.h"
 #include "altitude.h"
@@ -383,6 +384,16 @@ class GpsNmea : public QObject
      */
     void slot_reset();
 
+    /**
+     * called to open the NMEA log file.
+     */
+    void slot_openNmeaLogFile();
+
+    /**
+     * called to close the NMEA log file.
+     */
+    void slot_closeNmeaLogFile();
+
 #if 0
     /**
      * This slot is called to reset the gps device to factory
@@ -683,6 +694,8 @@ class GpsNmea : public QObject
     QString gpsDevice;
     /** reference to the normal serial connection */
     GpsCon* serial;
+    /** NMEA log file */
+    QFile* nmeaLogFile;
 
 #ifdef FLARM
 
