@@ -6,7 +6,7 @@
  ported to Qt4.x/X11  : (C) 2007-2011 by Axel Pauli
  maintainer           : axel@kflog.org
 
- $Id: mainwindow.cpp 4619 2011-05-04 19:32:17Z axel $
+ $Id: mainwindow.cpp 4685 2011-09-21 16:52:31Z axel $
 
 ****************************************************************************/
 
@@ -24,8 +24,8 @@
  *  and to initiate the load of the map and all other data.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
@@ -505,6 +505,9 @@ void MainWindow::slotCreateApplicationWidgets()
            this, SLOT( slotSwitchToInfoView( Waypoint* ) ) );
   connect( viewMap->_theMap, SIGNAL( airspaceWarning( const QString&, const bool ) ),
            this, SLOT( slotAlarm( const QString&, const bool ) ) );
+  connect( viewMap->_theMap, SIGNAL( newPosition( QPoint& ) ),
+           calculator, SLOT( slot_changePosition( QPoint& ) ) );
+
   connect( viewMap, SIGNAL( toggleLDCalculation( const bool ) ),
            calculator, SLOT( slot_toggleLDCalculation(const bool) ) );
   connect( viewMap, SIGNAL( toggleMenu() ), this, SLOT( slotToggleMenu() ) );
@@ -1994,7 +1997,7 @@ void MainWindow::slotReadconfig()
 /** Called if the status of the GPS changes, and controls the availability of manual navigation. */
 void MainWindow::slotGpsStatus( GpsNmea::GpsStatus status )
 {
-  playSound("notify");
+  static bool onePlay = false;
 
   if ( ( status != GpsNmea::validFix || calculator->isManualInFlight()) && ( view == mapView ) )
     {  // no GPS data
@@ -2003,6 +2006,16 @@ void MainWindow::slotGpsStatus( GpsNmea::GpsStatus status )
     }
   else
     {  // GPS data valid
+      if( onePlay == false )
+        {
+          // Only the first connect is reported via sound after the startup.
+          // That shall prevent multiple notifications if the GPS receiver
+          // plays ping pong (have a fix, have not a fix...) That can be very
+          // annoying. The current GPS state is signaled optical now.
+          playSound("notify");
+          onePlay = true;
+        }
+
       toggleManualNavActions( false );
       toggleGpsNavActions( true );
     }
@@ -2030,7 +2043,6 @@ void MainWindow::slotCenterToWaypoint()
     }
 }
 
-
 /** Called if the user pressed V in mapview.
   * Adjusts the zoom factor so that the currently selected waypoint
   * is displayed as good as possible. */
