@@ -12,7 +12,7 @@
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: mapcontents.cpp 4579 2011-03-31 18:11:21Z axel $
+ **   $Id: mapcontents.cpp 4665 2011-08-27 16:13:57Z axel $
  **
  ***********************************************************************/
 
@@ -136,6 +136,7 @@ MapContents::MapContents(QObject* parent, WaitScreen* waitscreen) :
   WaypointCatalog wpCat;
   int ok;
   const char* format;
+  QString error;
 
   if( GeneralConfig::instance()->getWaypointFileFormat() == GeneralConfig::Binary )
     {
@@ -144,7 +145,7 @@ MapContents::MapContents(QObject* parent, WaitScreen* waitscreen) :
     }
   else
     {
-      ok = wpCat.readXml( "", &wpList );
+      ok = wpCat.readXml( "", &wpList, error );
       format = "xml";
     }
 
