@@ -9,7 +9,7 @@
 
     email                : axel@kflog.org
 
-    $Id: mapview.cpp 4642 2011-05-23 18:08:31Z axel $
+    $Id: mapview.cpp 4654 2011-06-16 19:48:05Z axel $
 
  ***************************************************************************/
 
@@ -745,30 +745,40 @@ void MapView::slot_Position(const QPoint& position, const int source)
 /** This slot is called if the status of the GPS changes. */
 void MapView::slot_GPSStatus(GpsNmea::GpsStatus status)
 {
+  QColor color;
+
   switch ( status )
     {
     case GpsNmea::validFix:
       slot_info( tr( "GPS new fix" ) );
+      color = Qt::green;
       break;
     case GpsNmea::noFix:
       slot_info(tr( "GPS fix lost" ) );
+      color = Qt::yellow;
       break;
     default:
       slot_info( tr( "GPS lost" ) );
+      color = Qt::red;
     }
 
-  if(status>GpsNmea::notConnected)
+  if( status != GpsNmea::notConnected )
     {
-      _statusGps->setText(tr("GPS"));
+      _statusGps->setText( tr( "GPS" ) );
     }
   else
     {
-      _statusGps->setText(tr("Man"));
-      _heading->setValue("-");
-      _speed->setValue("-");
-      _altitude->setValue("-");
+      _statusGps->setText( tr( "Man" ) );
+      _heading->setValue( "-" );
+      _speed->setValue( "-" );
+      _altitude->setValue( "-" );
     }
 
+  // Set the color background according to the current GPS status.
+  _statusGps->setAutoFillBackground( true );
+  _statusGps->setBackgroundRole( QPalette::Window );
+  _statusGps->setPalette( QPalette( color ) );
+
   // Only show glider symbol if the GPS is connected.
   _theMap->setShowGlider( status == GpsNmea::validFix );
 }
