@@ -7,24 +7,21 @@
  ************************************************************************
  **
  **   Copyright (c):  2005      by André Somers
- **                   2009-2010 by Axel Pauli
+ **                   2009-2011 by Axel Pauli
  **
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: openairparser.cpp 4303 2010-10-08 19:25:18Z axel $
+ **   $Id: openairparser.cpp 4581 2011-04-01 19:11:40Z axel $
  **
  ***********************************************************************/
 
-// standard library includes
 #include <math.h>
 #include <stdlib.h>
 #include <unistd.h>
 
-//Qt includes
 #include <QtCore>
 
-//project includes
 #include "resource.h"
 #include "airspace.h"
 #include "openairparser.h"
@@ -46,7 +43,7 @@
 #define FILE_TYPE_AIRSPACE_C 0x61
 
 // version used for files created from OpenAir data
-#define FILE_VERSION_AIRSPACE_C 200
+#define FILE_VERSION_AIRSPACE_C 202
 
 extern MapContents*  _globalMapContents;
 extern MapMatrix*    _globalMapMatrix;
@@ -62,7 +59,6 @@ OpenAirParser::OpenAirParser()
   initializeBaseMapping();
 }
 
-
 OpenAirParser::~OpenAirParser()
 {
   if ( h_projection )
@@ -89,31 +85,31 @@ uint OpenAirParser::load( QList<Airspace*>& list )
   QStringList mapDirs = GeneralConfig::instance()->getMapDirectories();
   QStringList preselect;
 
-  for ( int i = 0; i < mapDirs.size(); ++i )
+  for( int i = 0; i < mapDirs.size(); ++i )
     {
-      MapContents::addDir(preselect, mapDirs.at(i) + "/airspaces", "*.txt");
-      MapContents::addDir(preselect, mapDirs.at(i) + "/airspaces", "*.TXT");
-      MapContents::addDir(preselect, mapDirs.at(i) + "/airspaces", "*.txc");
+      MapContents::addDir( preselect, mapDirs.at( i ) + "/airspaces", "*.txt" );
+      MapContents::addDir( preselect, mapDirs.at( i ) + "/airspaces", "*.TXT" );
+      MapContents::addDir( preselect, mapDirs.at( i ) + "/airspaces", "*.txc" );
     }
 
-  if (preselect.count() == 0)
+  if( preselect.count() == 0 )
     {
-      qWarning( "OpenAirParser: No Open Air files could be found in the map directories" );
+      qWarning( "OpenAirParser: No Open Air files found in the map directories!" );
       return loadCounter;
     }
 
   // First check, if we have found a file name in upper letters. May
-  // be true, if a file was downloaded from the internet. We will convert
+  // be true, if a file was downloaded from the Internet. We will convert
   // such a file name to lower cases and replace it in the file list.
-  for ( int i = 0; i < preselect.size(); ++i )
+  for( int i = 0; i < preselect.size(); ++i )
     {
-      if ( preselect.at(i).endsWith( ".TXT" ) )
+      if( preselect.at( i ).endsWith( ".TXT" ) )
         {
-          QFileInfo fInfo = preselect.at(i);
-          QString path    = fInfo.absolutePath();
-          QString fn      = fInfo.fileName().toLower();
-          QString newFn   = path + "/" + fn;
-          QFile::rename( preselect.at(i), newFn );
+          QFileInfo fInfo = preselect.at( i );
+          QString path = fInfo.absolutePath();
+          QString fn = fInfo.fileName().toLower();
+          QString newFn = path + "/" + fn;
+          QFile::rename( preselect.at( i ), newFn );
           preselect[i] = newFn;
         }
     }
@@ -121,19 +117,42 @@ uint OpenAirParser::load( QList<Airspace*>& list )
   // source files follows compiled files
   preselect.sort();
 
-  while (!preselect.isEmpty())
+  // Check, which files shall be loaded.
+  QStringList& files = GeneralConfig::instance()->getAirspaceFileList();
+
+  if( files.isEmpty() )
+    {
+      // No files shall be loaded
+      qWarning() << "OpenAirParser: No Open Air files defined for loading!";
+      return loadCounter;
+    }
+
+  if( files.first() != "All" )
+    {
+      for( int i = preselect.size() - 1; i >= 0; i-- )
+        {
+          QString file = QFileInfo(preselect.at(i)).completeBaseName() + ".txt";
+
+          if( files.contains( file ) == false )
+            {
+              preselect.removeAt(i);
+            }
+        }
+    }
+
+  while( !preselect.isEmpty() )
     {
       QString txtName, txcName;
       _doCompile = false;
 
-      if (preselect.first().endsWith(QString(".txt")))
+      if( preselect.first().endsWith( QString( ".txt" ) ) )
         {
           // there can't be the same name txc after this txt
           // parse found txt file
           txtName = preselect.first();
           _doCompile = true;
 
-          if (parse(txtName, list))
+          if( parse( txtName, list ) )
             {
               loadCounter++;
             }
@@ -166,8 +185,10 @@ uint OpenAirParser::load( QList<Airspace*>& list )
               unlink( txcName.toLatin1().data() );
               _doCompile = true;
 
-              if ( parse( txtName, list ) )
-                loadCounter++;
+              if( parse( txtName, list ) )
+                {
+                  loadCounter++;
+                }
 
               preselect.removeAt( 0 );
               continue;
@@ -187,8 +208,12 @@ uint OpenAirParser::load( QList<Airspace*>& list )
               // source file.
               unlink( txcName.toLatin1().data() );
               _doCompile = true;
-              if ( parse( txtName, list ) )
-                loadCounter++;
+
+              if( parse( txtName, list ) )
+                {
+                  loadCounter++;
+                }
+
               continue;
             }
 
@@ -196,8 +221,8 @@ uint OpenAirParser::load( QList<Airspace*>& list )
           QString confName = fi.path() + "/" + fi.baseName() + "_mappings.conf";
           QFileInfo fiConf( confName );
 
-          if ( fiConf.exists() && fi.isReadable() &&
-               h_creationDateTime < fiConf.lastModified() )
+          if( fiConf.exists() && fi.isReadable() &&
+              h_creationDateTime < fiConf.lastModified() )
             {
               // Conf file was modified, make a new compilation. It is not
               // deeper checked, what was modified due to the effort and
@@ -205,8 +230,12 @@ uint OpenAirParser::load( QList<Airspace*>& list )
               // every minute.
               unlink( txcName.toLatin1().data() );
               _doCompile = true;
+
               if ( parse( txtName, list ) )
-                loadCounter++;
+                {
+                  loadCounter++;
+                }
+
               continue;
             }
 
@@ -224,16 +253,22 @@ uint OpenAirParser::load( QList<Airspace*>& list )
 
               unlink( txcName.toLatin1().data() );
               _doCompile = true;
-              if ( parse( txtName, list ) )
-                loadCounter++;
+
+              if( parse( txtName, list ) )
+                {
+                  loadCounter++;
+                }
+
               continue;
             }
         }
 
       // we will read the compiled file, because a source file is not to
       // find after it or all checks were successfully passed
-      if ( readCompiledFile( txcName, list ) )
-        loadCounter++;
+      if( readCompiledFile( txcName, list ) )
+        {
+          loadCounter++;
+        }
 
     } // End of While
 
@@ -253,10 +288,12 @@ bool OpenAirParser::parse(const QString& path, QList<Airspace*>& list)
       return false;
     }
 
+  qDebug() << "OpenAirParser: Reading" << path;
+
   resetState();
   initializeStringMapping( path );
 
-  if ( _doCompile )
+  if( _doCompile )
     {
       // we open a buffer for temporary storage of extracted airspace
       // elements
@@ -274,43 +311,40 @@ bool OpenAirParser::parse(const QString& path, QList<Airspace*>& list)
     }
 
   QTextStream in(&source);
-  //start parsing
-  QString line=in.readLine();
-  _lineNumber++;
+  in.setCodec( "ISO 8859-15" );
 
-  while (!line.isNull())
+  while( ! in.atEnd() )
     {
+      QString line = in.readLine();
+      _lineNumber++;
+
       // qDebug("reading line %d: '%s'", _lineNumber, line.toLatin1().data());
       line = line.simplified();
 
-      if (line.startsWith("*") || line.startsWith("#"))
+      if( line.startsWith( "*" ) || line.startsWith( "#" ) || line.isEmpty() )
         {
-          //comment, ignore
+          continue;
         }
-      else if (line.isEmpty())
-        {
-          //empty line, ignore
-        }
-      else
-        {
-          // delete comments at the end of the line before parsing it
-          line = line.split('*')[0];
-          line = line.split('#')[0];
-          parseLine(line);
-        }
-      line=in.readLine();
-      _lineNumber++;
+
+      // delete comments at the end of the line before parsing it
+      line = line.split( '*' )[0];
+      line = line.split( '#' )[0];
+
+      parseLine( line );
     }
 
-  if (_isCurrentAirspace)
-    finishAirspace();
-
-  for (int i  = 0; i < _airlist.count(); i++)
+  if( _isCurrentAirspace )
     {
-      list.append(_airlist.at(i));
+      finishAirspace();
+    }
+
+  for( int i = 0; i < _airlist.count(); i++ )
+    {
+      list.append( _airlist.at( i ) );
     }
 
   QFileInfo fi( path );
+
   qDebug( "OpenAirParser: %d airspace objects read from file %s in %dms",
           _objCounter, fi.fileName().toLatin1().data(), t.elapsed() );
 
@@ -390,36 +424,47 @@ void OpenAirParser::resetState()
   _lineNumber = 0;
   _objCounter = 0;
   _isCurrentAirspace = false;
+  _acRead = false;
+  _anRead = false;
 }
 
 
 void OpenAirParser::parseLine(QString& line)
 {
+  if( (line.startsWith( "AC " ) || line.startsWith( "AN " )) &&
+       _acRead == true && _anRead == true )
+    {
+      // This indicates we're starting a new object and have to save the
+      // the previous one.
+      if( _isCurrentAirspace )
+        {
+          finishAirspace();
+        }
+
+      newAirspace();
+    }
+
   if (line.startsWith("AC "))
     {
-      //type of record. This also indicates we're starting a new object
-      if (_isCurrentAirspace)
-        finishAirspace();
-      newAirspace();
+      // airspace class
+      _acRead = true;
       parseType(line);
       return;
     }
 
-  //the rest of the records don't make sense if we're not parsing an object
-  int lat, lon;
-  double radius;
-  bool ok;
-
-  if (!_isCurrentAirspace)
-    return;
-
   if (line.startsWith("AN "))
     {
-      //name
+      // airspace name
+      _anRead = true;
       asName = line.mid(3);
       return;
     }
 
+  if (!_isCurrentAirspace)
+    {
+      return;
+    }
+
   if (line.startsWith("AH "))
     {
       //airspace ceiling
@@ -438,18 +483,23 @@ void OpenAirParser::parseLine(QString& line)
 
   if (line.startsWith("DP "))
     {
+      int lat, lon;
+
       //polygon coordinate
       QString coord = line.mid(3);
       parseCoordinate(coord, lat, lon);
-      asPA.append( QPoint(lat, lon) );
+      asPA.append(QPoint(lat, lon));
       // qDebug( "addDP: lat=%d, lon=%d", lat, lon );
       return;
     }
 
   if (line.startsWith("DC "))
     {
+      bool ok;
+
       //circle
-      radius=line.mid(3).toDouble(&ok);
+      double radius = line.mid(3).toDouble(&ok);
+
       if (ok)
         {
           addCircle(radius);
@@ -520,14 +570,15 @@ void OpenAirParser::newAirspace()
 {
   asName = "(unnamed)";
   asType = BaseMapElement::NotSelected;
-  //asTypeLetter = "";
   asPA.clear();
   asUpper = BaseMapElement::NotSet;
   asUpperType = BaseMapElement::NotSet;
   asLower = BaseMapElement::NotSet;
   asLowerType = BaseMapElement::NotSet;
   _isCurrentAirspace = true;
-  _direction = 1; //must be reset according to specifications
+  _acRead = false;
+  _anRead = false;
+  _direction = 1; // must be reset according to specifications
 }
 
 
@@ -541,6 +592,19 @@ void OpenAirParser::finishAirspace()
 {
   extern MapMatrix * _globalMapMatrix;
 
+  _isCurrentAirspace = false;
+  _acRead = false;
+  _anRead = false;
+
+  if( asPA.count() < 2 )
+    {
+      qWarning() << "OpenAirParser: Line" << _lineNumber
+                 << "Object" << asName
+                 << "contains to less coordinates! Ignoring it.";
+
+      return;
+    }
+
   // @AP: Airspaces are stored as polygons and should not contain the start point
   // twice as done in OpenAir description.
   if ( asPA.count() > 2 && asPA.first() == asPA.last() )
@@ -564,7 +628,7 @@ void OpenAirParser::finishAirspace()
                                asLower, asLowerType );
   _airlist.append(as);
   _objCounter++;
-  _isCurrentAirspace = false;
+
   // qDebug("finalized airspace %s. %d points in airspace", asName.toLatin1().data(), cnt);
 
   if ( _doCompile )
@@ -646,7 +710,8 @@ void OpenAirParser::initializeStringMapping(const QString& mapFilePath)
       QFile f(path);
       if (!f.open(QIODevice::ReadOnly))
         {
-          qWarning("OpenAirParser: Cannot open airspace mapping file %s!", path.toLatin1().data());
+          qWarning("OpenAirParser: Cannot open airspace mapping file %s!",
+                   path.toLatin1().data());
           return;
         }
 
@@ -702,7 +767,8 @@ void OpenAirParser::parseType(QString& line)
       if (!m_baseTypeMap.contains(stringType))
         {
           //the indicated basetype is not a valid Cumulus basetype.
-          qWarning("OpenAirParser:=Line %d, Type '%s' is not a valid basetype. Object not interpretted.", _lineNumber, stringType.toLatin1().data());
+          qWarning( "OpenAirParser: Line=%d, Type '%s' is not a valid basetype. Object not interpreted.",
+                    _lineNumber, stringType.toLatin1().data());
           _isCurrentAirspace = false; //stop accepting other lines in this object
           return;
         }
@@ -717,20 +783,20 @@ void OpenAirParser::parseType(QString& line)
 
 void OpenAirParser::parseAltitude(QString& line, BaseMapElement::elevationType& type, int& alt)
 {
-  bool convertFromMeters=false;
-  bool altitudeIsFeet=false;
+  bool convertFromMeters = false;
+  bool altitudeIsFeet = false;
   QString input = line;
   QStringList elements;
-  int len=0, pos=0;
+  int len = 0, pos = 0;
   QString part;
   bool ok;
-  int num=0;
+  int num = 0;
 
   type = BaseMapElement::NotSet;
-  alt=0;
+  alt = 0;
+
   // qDebug("line %d: parsing altitude '%s'", _lineNumber, line.toLatin1().data());
-  //fist, split the string in parsable elements
-  //we start with the text parts
+  // Fist, split the string in parsable elements. We start with the text parts.
   QRegExp reg("[A-Za-z]+");
 
   while (line.length()>0)
@@ -742,44 +808,40 @@ void OpenAirParser::parseAltitude(QString& line, BaseMapElement::elevationType&
           break;
         }
       elements.append(line.mid(pos, len));
-      //qDebug("element: '%s'", line.mid(pos, len).toLatin1().data());
-      //line=line.mid(len);
     }
 
-  //now, get our number parts
+  // now, get our number parts
 
   reg.setPattern("[0-9]+");
   pos=0;
   len=0;
 
-  while (line.length()>0)
+  while (line.length() > 0)
     {
       pos = reg.indexIn(line, pos+len);
       len = reg.matchedLength();
 
-      if (pos<0)
+      if (pos < 0)
         {
           break;
         }
+
       elements.append(line.mid(pos, len));
-      //qDebug("element: '%s'", line.mid(pos, len).toLatin1().data());
-      line=line.mid(len);
+      line = line.mid(len);
     }
 
   // now, try parsing piece by piece
-  // print it out
-
   for ( QStringList::Iterator it = elements.begin(); it != elements.end(); ++it )
     {
       part = (*it).toUpper();
       BaseMapElement::elevationType newType = BaseMapElement::NotSet;
 
       // first, try to interpret as elevation type
-      if (part=="AMSL" || part=="MSL")
+      if ( part == "AMSL" || part == "MSL" || part == "ALT" )
         {
           newType=BaseMapElement::MSL;
         }
-      else if (part=="GND" || part=="SFC" || part=="ASFC" || part=="AGL")
+      else if ( part == "GND" || part == "SFC" || part == "ASFC" || part == "AGL" || part == "GROUND" )
         {
           newType=BaseMapElement::GND;
         }
@@ -812,29 +874,31 @@ void OpenAirParser::parseAltitude(QString& line, BaseMapElement::elevationType&
           continue;
         }
 
-      //see if it is a way of setting units to feet
-      if (part=="FT")
+      // see if it is a way of setting units to feet
+      if (part == "FT")
         {
-          altitudeIsFeet=true;
+          altitudeIsFeet = true;
           continue;
         }
 
-      //see if it is a way of setting units to meters
-      if (part=="M")
+      // see if it is a way of setting units to meters
+      if (part == "M")
         {
-          convertFromMeters=true;
+          convertFromMeters = true;
           continue;
         }
 
-      //try to interpret as a number
+      // try to interpret as a number
       num = part.toInt(&ok);
+
       if (ok)
         {
           alt = num;
         }
 
-      //ignore other parts
+      // ignore other parts
     }
+
   if ( altitudeIsFeet && type == BaseMapElement::NotSet )
     {
       type = BaseMapElement::MSL;
@@ -849,7 +913,7 @@ void OpenAirParser::parseAltitude(QString& line, BaseMapElement::elevationType&
     {
       // @AP: Altitude is zero but no type is assigned. In this case GND
       // is assumed. Found that in a polish airspace file.
-      type=BaseMapElement::GND;
+      type = BaseMapElement::GND;
     }
 
   // qDebug("Line %d: Returned altitude %d, type %d", _lineNumber, alt, int(type));
@@ -1298,6 +1362,8 @@ bool OpenAirParser::readCompiledFile( QString &path, QList<Airspace*>& list )
       return false;
     }
 
+  qDebug() << "OpenAirParser: Reading" << path;
+
   QDataStream in(&inFile);
 
   // This was the order used by ealier cumulus
@@ -1359,11 +1425,13 @@ bool OpenAirParser::readCompiledFile( QString &path, QList<Airspace*>& list )
   in >> boundingBox;
 #endif
 
-  // because we're used a buffer during output, the buffer length will
-  // be written too. Now we have to read it but don't need it because
-  // we access directly to the opened file.
+  // Because we're used a buffer during output, the buffer length will
+  // be written too. We have to read it and make a preallocation in the
+  // airspace list.
   in >> buflen;
 
+  list.reserve( list.size() + buflen );
+
   uint counter = 0;
 
   QString name;
